@@ -49,7 +49,9 @@ cmake .. -DCMAKE_C_COMPILER=gcc \
          -DCMAKE_CXX_COMPILER=g++ \
          -DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE} \
          ${EXTRA_CMAKE_FLAGS} -DINSTALL_GTEST='' -GNinja
-ninja
+[[ "$OSTYPE" == "darwin"* ]] && NUM_JOBS="$(sysctl -n hw.ncpu)" || NUM_JOBS="$(nproc --all)"
+echo "Starting ninja build with up to ${NUM_JOBS} parallel jobs..."
+ninja -j ${NUM_JOBS}
 
 if [ -n "${MAKE_DEB}" ]
 then
