@@ -1,7 +1,7 @@
 /*
  * ct_actions_find.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2022
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -102,7 +102,7 @@ void CtActions::find_in_selected_node()
         if (not all_matches) break;
     }
     if (0 == _s_state.matches_num) {
-        CtDialogs::info_dialog(str::format(_("The pattern '%s' was not found"), pattern), *_pCtMainWin);
+        CtDialogs::info_dialog(str::format(_("The pattern '%s' was not found"), str::xml_escape(pattern)), *_pCtMainWin);
     }
     else if (all_matches) {
         _s_state.match_dialog_title = std::to_string(_s_state.matches_num) + CtConst::CHAR_SPACE + _("Matches");
@@ -233,7 +233,7 @@ void CtActions::find_in_multiple_nodes()
 
     _pCtMainWin->user_active() = user_active_restore;
     auto last_iterated_node = _pCtMainWin->curr_tree_iter();
-    ctTreeStore.treeview_set_tree_expanded_collapsed_string(tree_expanded_collapsed_string, ctTreeView, _pCtMainWin->get_ct_config()->nodesBookmExp);
+    ctTreeStore.treeview_set_tree_expanded_collapsed_string(tree_expanded_collapsed_string, ctTreeView, _pCtConfig->nodesBookmExp);
     if (not _s_state.matches_num or all_matches) {
         ctTreeView.set_cursor_safe(starting_tree_iter);
         ctTextView.grab_focus();
@@ -241,7 +241,7 @@ void CtActions::find_in_multiple_nodes()
         ctTextView.scroll_to(curr_buffer->get_insert(), CtTextView::TEXT_SCROLL_MARGIN);
     }
     if (not _s_state.matches_num) {
-        CtDialogs::info_dialog(str::format(_("The pattern '%s' was not found"), std::string(pattern)), *_pCtMainWin);
+        CtDialogs::info_dialog(str::format(_("The pattern '%s' was not found"), str::xml_escape(pattern)), *_pCtMainWin);
     }
     else {
         if (all_matches) {
@@ -395,6 +395,10 @@ bool CtActions::_parse_node_name_n_tags_iter(CtTreeIter& node_iter,
                                              Glib::RefPtr<Glib::Regex> re_pattern,
                                              const bool all_matches)
 {
+    if (not _is_node_within_time_filter(node_iter)) {
+        return false;
+    }
+
     Glib::ustring node_name = node_iter.get_node_name();
     Glib::MatchInfo match;
     if (not re_pattern->match(node_name, match)) {
@@ -434,6 +438,10 @@ bool CtActions::_parse_node_content_iter(const CtTreeIter& tree_iter,
                                          bool all_matches,
                                          bool first_node)
 {
+    if (not _is_node_within_time_filter(tree_iter)) {
+        return false;
+    }
+
     bool restore_modified;
     Gtk::TextIter buff_start_iter = text_buffer->begin();
     if (buff_start_iter.get_char() != '\n') {
@@ -455,9 +463,7 @@ bool CtActions::_parse_node_content_iter(const CtTreeIter& tree_iter,
         if (all_matches) _s_state.all_matches_first_in_node = false;
     }
 
-    bool pattern_found{false};
-    if (_is_node_within_time_filter(tree_iter))
-        pattern_found = _find_pattern(tree_iter, text_buffer, re_pattern, start_iter, forward, all_matches);
+    bool pattern_found = _find_pattern(tree_iter, text_buffer, re_pattern, start_iter, forward, all_matches);
 
     if (_s_state.newline_trick) {
         buff_start_iter = text_buffer->begin();
@@ -538,7 +544,7 @@ Glib::RefPtr<Glib::Regex> CtActions::_create_re_pattern(Glib::ustring pattern)
             return Glib::Regex::create(pattern, Glib::RegexCompileFlags::REGEX_MULTILINE | Glib::RegexCompileFlags::REGEX_CASELESS);
     }
     catch (Glib::RegexError& e) {
-        CtDialogs::error_dialog(e.what(), *_pCtMainWin);
+        CtDialogs::error_dialog(str::xml_escape(e.what()), *_pCtMainWin);
         return Glib::RefPtr<Glib::Regex>();
     }
 }
