@@ -1,7 +1,7 @@
 /*
  * ct_dialogs.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2022
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -398,7 +398,7 @@ void CtDialogs::dialog_about(Gtk::Window& parent, Glib::RefPtr<Gdk::Pixbuf> icon
     auto dialog = Gtk::AboutDialog();
     dialog.set_program_name("CherryTree");
     dialog.set_version(CtConst::CT_VERSION);
-    dialog.set_copyright("Copyright © 2009-2021\n"
+    dialog.set_copyright("Copyright © 2009-2022\n"
                          "Giuseppe Penone <giuspen@gmail.com>\n"
                          "Evgenii Gurianov <https://github.com/txe>");
     dialog.set_comments(_("A Hierarchical Note Taking Application, featuring Rich Text and Syntax Highlighting"));
@@ -422,17 +422,20 @@ MA 02110-1301, USA.
     dialog.set_authors({"Giuseppe Penone <giuspen@gmail.com>", "Evgenii Gurianov <https://github.com/txe>"});
     dialog.set_artists({"Ugo Yak <https://www.instagram.com/ugoyak.art/>", "OCAL <http://www.openclipart.org/>", "Zeltak <zeltak@gmail.com>", "Angelo Penone <angelo.penone@gmail.com>"});
     dialog.set_translator_credits(Glib::ustring{} +
+ _("Arabic")+" (ar) Mohamed Milodi <milodi500@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Armenian")+" (hy) Seda Stamboltsyan <sedastam@yandex.com>"+CtConst::CHAR_NEWLINE+
  _("Bulgarian")+" (bg) Iliya Nikolaev <iliya.nikolaev@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Chinese Simplified")+" (zh_CN) Wang Yu <krwy0330@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Chinese Traditional")+" (zh_TW) Emer Chen <ldsemerchen@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Croatian")+" (hr) Filip Bakula <filip.test46@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Czech")+" (cs) Pavel Fric <fripohled@blogspot.com>"+CtConst::CHAR_NEWLINE+
- _("Dutch")+" (nl) Luuk Geurts, Patrick Vijgeboom <pj.vijgeboom@gmail.com>"+CtConst::CHAR_NEWLINE+
+ _("Dutch")+" (nl) Luuk Geurts"+CtConst::CHAR_NEWLINE+
  _("Finnish")+" (fi) Henri Kaustinen <hendrix.ks81@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("French")+" (fr) Klaus Becker <colonius@free.fr>"+CtConst::CHAR_NEWLINE+
  _("German")+" (de) Matthias Hoffmann <MHoffi@web.de>"+CtConst::CHAR_NEWLINE+
  _("Greek")+" (el) Delphina <delphina.2009@yahoo.gr>"+CtConst::CHAR_NEWLINE+
+ _("Hindi India")+" (hi_IN) Paresh Mhatre <pareshmhatrework@gmail.com>"+CtConst::CHAR_NEWLINE+
+ _("Hungarian")+" (hu) Stiener Norbert <stienernorbert@protonmail.com>"+CtConst::CHAR_NEWLINE+
  _("Italian")+" (it) Vincenzo Reale <smart2128vr@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Japanese")+" (ja) Piyo <py2@live.jp>"+CtConst::CHAR_NEWLINE+
  _("Kazakh")+" (kk_KZ) Viktor Polyanskiy <camilot55@yandex.ru>"+CtConst::CHAR_NEWLINE+
@@ -495,26 +498,31 @@ void CtDialogs::summary_info_dialog(CtMainWin* pCtMainWin, const CtSummaryInfo&
     grid.attach(label_im_key, 0, 3, 1, 1);
     Gtk::Label label_im_val{std::to_string(summaryInfo.images_num)};
     grid.attach(label_im_val, 1, 3, 1, 1);
+    Gtk::Label label_lx_key;
+    label_lx_key.set_markup(Glib::ustring{"<b>"} + _("Number of LatexBoxes") + "</b>");
+    grid.attach(label_lx_key, 0, 4, 1, 1);
+    Gtk::Label label_lx_val{std::to_string(summaryInfo.latexes_num)};
+    grid.attach(label_lx_val, 1, 4, 1, 1);
     Gtk::Label label_ef_key;
     label_ef_key.set_markup(Glib::ustring{"<b>"} + _("Number of Embedded Files") + "</b>");
-    grid.attach(label_ef_key, 0, 4, 1, 1);
+    grid.attach(label_ef_key, 0, 5, 1, 1);
     Gtk::Label label_ef_val{std::to_string(summaryInfo.embfile_num)};
-    grid.attach(label_ef_val, 1, 4, 1, 1);
+    grid.attach(label_ef_val, 1, 5, 1, 1);
     Gtk::Label label_ta_key;
     label_ta_key.set_markup(Glib::ustring{"<b>"} + _("Number of Tables") + "</b>");
-    grid.attach(label_ta_key, 0, 5, 1, 1);
+    grid.attach(label_ta_key, 0, 6, 1, 1);
     Gtk::Label label_ta_val{std::to_string(summaryInfo.tables_num)};
-    grid.attach(label_ta_val, 1, 5, 1, 1);
+    grid.attach(label_ta_val, 1, 6, 1, 1);
     Gtk::Label label_cb_key;
     label_cb_key.set_markup(Glib::ustring{"<b>"} + _("Number of CodeBoxes") + "</b>");
-    grid.attach(label_cb_key, 0, 6, 1, 1);
+    grid.attach(label_cb_key, 0, 7, 1, 1);
     Gtk::Label label_cb_val{std::to_string(summaryInfo.codeboxes_num)};
-    grid.attach(label_cb_val, 1, 6, 1, 1);
+    grid.attach(label_cb_val, 1, 7, 1, 1);
     Gtk::Label label_an_key;
     label_an_key.set_markup(Glib::ustring{"<b>"} + _("Number of Anchors") + "</b>");
-    grid.attach(label_an_key, 0, 7, 1, 1);
+    grid.attach(label_an_key, 0, 8, 1, 1);
     Gtk::Label label_an_val{std::to_string(summaryInfo.anchors_num)};
-    grid.attach(label_an_val, 1, 7, 1, 1);
+    grid.attach(label_an_val, 1, 8, 1, 1);
     Gtk::Box* pContentArea = dialog.get_content_area();
     pContentArea->pack_start(grid);
     pContentArea->show_all();
