@@ -1,7 +1,7 @@
 /*
  * ct_export2html.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2022
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -365,25 +365,23 @@ Glib::ustring CtExport2Html::_get_embfile_html(CtImageEmbFile* embfile, CtTreeIt
 // Returns the HTML Image
 Glib::ustring CtExport2Html::_get_image_html(CtImage* image, const fs::path& images_dir, int& images_count, CtTreeIter* tree_iter)
 {
-    if (CtImageAnchor* imageAnchor = dynamic_cast<CtImageAnchor*>(image))
+    if (CtImageAnchor* imageAnchor = dynamic_cast<CtImageAnchor*>(image)) {
         return "<a name=\"" + imageAnchor->get_anchor_name() + "\"></a>";
-
+    }
     images_count += 1;
     Glib::ustring image_name, image_rel_path;
-    if (tree_iter)
-    {
+    if (tree_iter) {
         image_name = std::to_string(tree_iter->get_node_id()) + "-" + std::to_string(images_count) + ".png";
         image_rel_path = (fs::path{"images"} / image_name).string_unix();
     }
-    else
-    {
+    else {
         image_name = std::to_string(images_count) + ".png";
         image_rel_path = "file://" + (images_dir / image_name).string_unix();
     }
 
     Glib::ustring image_html = "<img src=\"" + image_rel_path + "\" alt=\"" + image_rel_path + "\" />";
-    if (CtImagePng* png = dynamic_cast<CtImagePng*>(image))
-    {
+    CtImagePng* png = dynamic_cast<CtImagePng*>(image);
+    if (png and not png->get_link().empty()) {
         Glib::ustring href = _get_href_from_link_prop_val(png->get_link());
         image_html = "<a href=\"" + href + "\">" + image_html + "</a>";
     }
@@ -486,7 +484,7 @@ Glib::ustring CtExport2Html::_html_get_from_code_buffer(const Glib::RefPtr<Gsv::
         //html_text += str::replace(sym, " ", "&nbsp;");
         // let's try and use <pre></pre> instead of '&nbsp;' to preserve the spaces
         html_text += sym;
-        if (!curr_iter.forward_char() || (sel_end >= 0 && curr_iter.get_offset() > sel_end)) {
+        if (!curr_iter.forward_char() || (sel_end >= 0 && curr_iter.get_offset() >= sel_end)) {
             if (span_opened) html_text += "</span>";
             break;
         }
