@@ -1,7 +1,7 @@
 /*
  * ct_export2pdf.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2022
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -30,6 +30,12 @@ namespace {
     Glib::ustring generate_tag(NodeIdType node_id, const Glib::ustring& anchor) { return fmt::format("n.{}.{}", node_id, anchor); }
 }
 
+CtExport2Pango::CtExport2Pango(CtMainWin* pCtMainWin)
+ : _pCtMainWin{pCtMainWin}
+ , _pCtConfig{pCtMainWin->get_ct_config()}
+{
+}
+
 // Given a treestore iter returns the Pango rich text
 void CtExport2Pango::pango_get_from_treestore_node(CtTreeIter node_iter, int sel_start, int sel_end, std::vector<CtPangoObjectPtr>& out_slots)
 {
@@ -63,36 +69,40 @@ void CtExport2Pango::pango_get_from_treestore_node(CtTreeIter node_iter, int sel
 }
 
 // Get rich text from syntax highlighted code node
-Glib::ustring CtExport2Pango::pango_get_from_code_buffer(Glib::RefPtr<Gsv::Buffer> code_buffer, int sel_start, int sel_end, const std::string& syntax_highlighting)
+Glib::ustring CtExport2Pango::pango_get_from_code_buffer(Glib::RefPtr<Gsv::Buffer> code_buffer,
+                                                         int sel_start,
+                                                         int sel_end,
+                                                         const std::string& syntax_highlighting)
 {
     Gtk::TextIter curr_iter = sel_start < 0 ? code_buffer->begin() : code_buffer->get_iter_at_offset(sel_start);
     Gtk::TextIter end_iter = sel_start < 0 ? code_buffer->end() : code_buffer->get_iter_at_offset(sel_end);
-    _pCtMainWin->apply_syntax_highlighting(code_buffer, syntax_highlighting, false/*forceReApply*/);
-    code_buffer->ensure_highlight(curr_iter, end_iter);
-    Glib::ustring pango_text = "";
+    Glib::ustring indentation;
+    bool indentation_force_spaces{false};
+    if (syntax_highlighting != CtConst::PLAIN_TEXT_ID) {
+        _pCtMainWin->apply_syntax_highlighting(code_buffer, syntax_highlighting, false/*forceReApply*/);
+        code_buffer->ensure_highlight(curr_iter, end_iter);
+        indentation = str::repeat(CtConst::CHAR_SPACE, _pCtConfig->tabsWidth);
+        indentation_force_spaces = true;
+    }
+    Glib::ustring pango_text;
     Glib::ustring former_tag_str = CtConst::COLOR_48_BLACK;
-    bool span_opened = false;
-    while (true)
-    {
+    bool span_opened{false};
+    bool is_indentation{true};
+    while (true) {
         auto curr_tags = curr_iter.get_tags();
-        if (!curr_tags.empty())
-        {
+        if (not curr_tags.empty()) {
             Glib::ustring curr_tag_str = curr_tags[0]->property_foreground_gdk().get_value().to_string();
             int font_weight = curr_tags[0]->property_weight();
-            if (curr_tag_str == CtConst::COLOR_48_BLACK)
-            {
-                if (former_tag_str != curr_tag_str)
-                {
+            if (curr_tag_str == CtConst::COLOR_48_BLACK) {
+                if (former_tag_str != curr_tag_str) {
                     former_tag_str = curr_tag_str;
                     // end of tag
                     pango_text += "</span>";
                     span_opened = false;
                 }
             }
-            else
-            {
-                if (former_tag_str != curr_tag_str)
-                {
+            else {
+                if (former_tag_str != curr_tag_str) {
                     former_tag_str = curr_tag_str;
                     if (span_opened) pango_text += "</span>";
                     // start of tag
@@ -102,15 +112,32 @@ Glib::ustring CtExport2Pango::pango_get_from_code_buffer(Glib::RefPtr<Gsv::Buffe
                 }
             }
         }
-        else if (span_opened)
-        {
+        else if (span_opened) {
             span_opened = false;
             former_tag_str = CtConst::COLOR_48_BLACK;
             pango_text += "</span>";
         }
-        pango_text += str::xml_escape(Glib::ustring(1, curr_iter.get_char()));
-        if (!curr_iter.forward_char() || (sel_end >= 0 && curr_iter.get_offset() > sel_end))
-        {
+        const auto curr_char = curr_iter.get_char();
+        if (indentation_force_spaces) {
+            if (is_indentation) {
+                if ('\t' == curr_char) {
+                    pango_text += indentation;
+                }
+                else {
+                    is_indentation = false;
+                }
+            }
+            if (not is_indentation) {
+                pango_text += str::xml_escape(Glib::ustring{1, curr_char});
+                if ('\n' == curr_char) {
+                    is_indentation = true;
+                }
+            }
+        }
+        else {
+            pango_text += str::xml_escape(Glib::ustring{1, curr_char});
+        }
+        if (not curr_iter.forward_char() or (sel_end >= 0 && curr_iter.get_offset() > sel_end)) {
             if (span_opened) pango_text += "</span>";
             break;
         }
@@ -164,7 +191,6 @@ void CtExport2Pango::_pango_text_serialize(const Gtk::TextIter& start_iter,
                     subscript_active = true;
                     continue;
                 }
-                CtConfig* pConfig = _pCtMainWin->get_ct_config();
                 size_t sIdx;
                 if (property_value == CtConst::TAG_PROP_VAL_SMALL) sIdx = 6;
                 else if (property_value == CtConst::TAG_PROP_VAL_H1) sIdx = 0;
@@ -178,8 +204,8 @@ void CtExport2Pango::_pango_text_serialize(const Gtk::TextIter& start_iter,
                     continue;
                 }
                 tag_property = "font_size";
-                const auto fontSize = Pango::FontDescription{pConfig->rtFont}.get_size();
-                const auto pCtScalableTag = pConfig->scalablesTags.at(sIdx);
+                const auto fontSize = Pango::FontDescription{_pCtConfig->rtFont}.get_size();
+                const auto pCtScalableTag = _pCtConfig->scalablesTags.at(sIdx);
                 property_value = std::to_string(static_cast<unsigned>(fontSize * pCtScalableTag->scale));
                 // check if other optional configuration is applied to this scalable tag
                 if (not pCtScalableTag->foreground.empty()) {
@@ -200,15 +226,14 @@ void CtExport2Pango::_pango_text_serialize(const Gtk::TextIter& start_iter,
             }
             else if (tag_property == CtConst::TAG_FAMILY) {
                 tag_property = "font_family";
-                CtConfig* pConfig = _pCtMainWin->get_ct_config();
-                if (not pConfig->monospaceFg.empty()) {
-                    pango_attrs += std::string{" "} + CtConst::TAG_FOREGROUND + "=\"" + pConfig->monospaceFg + "\"";
+                if (not _pCtConfig->monospaceFg.empty()) {
+                    pango_attrs += std::string{" "} + CtConst::TAG_FOREGROUND + "=\"" + _pCtConfig->monospaceFg + "\"";
                 }
-                if (not pConfig->monospaceBg.empty()) {
-                    pango_attrs += std::string{" "} + CtConst::TAG_BACKGROUND + "=\"" + pConfig->monospaceBg + "\"";
+                if (not _pCtConfig->monospaceBg.empty()) {
+                    pango_attrs += std::string{" "} + CtConst::TAG_BACKGROUND + "=\"" + _pCtConfig->monospaceBg + "\"";
                 }
-                if (pConfig->msDedicatedFont and not pConfig->monospaceFont.empty()) {
-                    auto fontDesc = Pango::FontDescription{pConfig->monospaceFont};
+                if (_pCtConfig->msDedicatedFont and not _pCtConfig->monospaceFont.empty()) {
+                    auto fontDesc = Pango::FontDescription{_pCtConfig->monospaceFont};
                     property_value = fontDesc.get_family();
                     pango_attrs += std::string{" font_size=\""} + std::to_string(fontDesc.get_size()) + "\"";
                 }
@@ -288,14 +313,15 @@ void CtExport2Pdf::node_export_print(const fs::path& pdf_filepath, CtTreeIter tr
     if (tree_iter.get_node_is_rich_text())
         CtExport2Pango{_pCtMainWin}.pango_get_from_treestore_node(tree_iter, sel_start, sel_end, pango_slots);
     else {
-        Glib::ustring text = CtExport2Pango{_pCtMainWin}.pango_get_from_code_buffer(tree_iter.get_node_text_buffer(), sel_start, sel_end, tree_iter.get_node_syntax_highlighting());
+        Glib::ustring text = CtExport2Pango{_pCtMainWin}.pango_get_from_code_buffer(
+            tree_iter.get_node_text_buffer(), sel_start, sel_end, tree_iter.get_node_syntax_highlighting());
         pango_slots.push_back(std::make_shared<CtPangoText>(text, tree_iter.get_node_syntax_highlighting(), 0));
     }
 
     if (options.include_node_name)
         pango_slots.emplace(pango_slots.begin(), _generate_pango_node_name(tree_iter));
 
-    _pCtMainWin->get_ct_print().print_text(_pCtMainWin, pdf_filepath, pango_slots);
+    _pCtMainWin->get_ct_print().print_text(pdf_filepath, pango_slots);
 }
 
 void CtExport2Pdf::node_and_subnodes_export_print(const fs::path& pdf_filepath, CtTreeIter tree_iter, const CtExportOptions& options)
@@ -303,19 +329,17 @@ void CtExport2Pdf::node_and_subnodes_export_print(const fs::path& pdf_filepath,
     std::vector<CtPangoObjectPtr> tree_pango_slots;
     _nodes_all_export_print_iter(tree_iter, options, tree_pango_slots);
 
-    _pCtMainWin->get_ct_print().print_text(_pCtMainWin, pdf_filepath, tree_pango_slots);
+    _pCtMainWin->get_ct_print().print_text(pdf_filepath, tree_pango_slots);
 }
 
 void CtExport2Pdf::tree_export_print(const fs::path& pdf_filepath, CtTreeIter tree_iter, const CtExportOptions& options)
 {
     std::vector<CtPangoObjectPtr> tree_pango_slots;
-    Glib::ustring text_font = _pCtMainWin->get_ct_config()->codeFont;
-    while (tree_iter)
-    {
+    while (tree_iter) {
         _nodes_all_export_print_iter(tree_iter, options, tree_pango_slots);
         ++tree_iter;
     }
-    _pCtMainWin->get_ct_print().print_text(_pCtMainWin, pdf_filepath, tree_pango_slots);
+    _pCtMainWin->get_ct_print().print_text(pdf_filepath, tree_pango_slots);
 }
 
 void CtExport2Pdf::_nodes_all_export_print_iter(CtTreeIter tree_iter, const CtExportOptions& options, std::vector<CtPangoObjectPtr>& tree_pango_slots)
@@ -324,7 +348,8 @@ void CtExport2Pdf::_nodes_all_export_print_iter(CtTreeIter tree_iter, const CtEx
     if (tree_iter.get_node_is_rich_text())
         CtExport2Pango{_pCtMainWin}.pango_get_from_treestore_node(tree_iter, -1, -1, node_pango_slots);
     else {
-        Glib::ustring text = CtExport2Pango{_pCtMainWin}.pango_get_from_code_buffer(tree_iter.get_node_text_buffer(), -1, -1, tree_iter.get_node_syntax_highlighting());
+        Glib::ustring text = CtExport2Pango{_pCtMainWin}.pango_get_from_code_buffer(
+            tree_iter.get_node_text_buffer(), -1, -1, tree_iter.get_node_syntax_highlighting());
         node_pango_slots.push_back(std::make_shared<CtPangoText>(text, tree_iter.get_node_syntax_highlighting(), 0));
     }
 
@@ -353,12 +378,9 @@ CtPangoObjectPtr CtExport2Pdf::_generate_pango_node_name(CtTreeIter tree_iter)
     return slot;
 }
 
-
-
-
-
-
-CtPrint::CtPrint()
+CtPrint::CtPrint(CtMainWin* pCtMainWin)
+ : _pCtMainWin{pCtMainWin}
+ , _pCtConfig{pCtMainWin->get_ct_config()}
 {
     _pPrintSettings = Gtk::PrintSettings::create();
     _pPageSetup = Gtk::PageSetup::create();
@@ -392,10 +414,8 @@ void CtPrint::run_page_setup_dialog(Gtk::Window* pWin)
 }
 
 // Start the Print Operations for Text
-void CtPrint::print_text(CtMainWin* pCtMainWin, const fs::path& pdf_filepath, const std::vector<CtPangoObjectPtr>& slots)
+void CtPrint::print_text(const fs::path& pdf_filepath, const std::vector<CtPangoObjectPtr>& slots)
 {
-    _pCtMainWin = pCtMainWin;
-
     CtPrintData print_data;
     print_data.slots = slots;
 
@@ -409,17 +429,15 @@ void CtPrint::print_text(CtMainWin* pCtMainWin, const fs::path& pdf_filepath, co
     print_data.operation->signal_begin_print().connect(sigc::bind(fun_begin_print_text, &print_data));
     print_data.operation->signal_draw_page().connect(sigc::bind(fun_draw_page_text, &print_data));
     print_data.operation->set_export_filename(pdf_filepath.string());
-    try
-    {
+    try {
         auto res = print_data.operation->run(!pdf_filepath.empty() ? Gtk::PRINT_OPERATION_ACTION_EXPORT : Gtk::PRINT_OPERATION_ACTION_PRINT_DIALOG);
         if (res == Gtk::PRINT_OPERATION_RESULT_ERROR)
-            CtDialogs::error_dialog("Error printing file: (bad res)", *pCtMainWin);
+            CtDialogs::error_dialog("Error printing file: (bad res)", *_pCtMainWin);
         else if (res == Gtk::PRINT_OPERATION_RESULT_APPLY)
             _pPrintSettings = print_data.operation->get_print_settings();
     }
-    catch (Glib::Error& ex)
-    {
-        CtDialogs::error_dialog("Error printing file:\n" + ex.what() + " (exception caught)", *pCtMainWin);
+    catch (Glib::Error& ex) {
+        CtDialogs::error_dialog("Error printing file:\n" + str::xml_escape(ex.what()) + " (exception caught)", *_pCtMainWin);
     }
     if (!print_data.warning.empty())
         _pCtMainWin->get_status_bar().update_status(print_data.warning);
@@ -439,9 +457,9 @@ void CtPrint::_on_begin_print_text(const Glib::RefPtr<Gtk::PrintContext>& contex
     };
 
     print_data->context = context;
-    _rich_font = get_font_with_fallback_(Pango::FontDescription(_pCtMainWin->get_ct_config()->rtFont), _pCtMainWin->get_ct_config()->fallbackFontFamily);
-    _plain_font = get_font_with_fallback_(Pango::FontDescription(_pCtMainWin->get_ct_config()->ptFont), _pCtMainWin->get_ct_config()->fallbackFontFamily);
-    _code_font = get_font_with_fallback_(Pango::FontDescription(_pCtMainWin->get_ct_config()->codeFont), "monospace");
+    _rich_font = get_font_with_fallback_(Pango::FontDescription(_pCtConfig->rtFont), _pCtConfig->fallbackFontFamily);
+    _plain_font = get_font_with_fallback_(Pango::FontDescription(_pCtConfig->ptFont), _pCtConfig->fallbackFontFamily);
+    _code_font = get_font_with_fallback_(Pango::FontDescription(_pCtConfig->codeFont), "monospace");
     _text_window_width = _pCtMainWin->get_text_view().get_allocation().get_width();
     _table_text_row_height = _rich_font.get_size()/Pango::SCALE;
     _table_line_thickness = 6;
@@ -684,7 +702,8 @@ void CtPrint::_process_pango_codebox(CtPrintData* print_data, CtCodebox* codebox
     auto context = print_data->context;
     CtPrintPages& pages = print_data->pages;
 
-    Glib::ustring original_content = CtExport2Pango{_pCtMainWin}.pango_get_from_code_buffer(codebox->get_buffer(), -1, -1, codebox->get_syntax_highlighting());
+    Glib::ustring original_content = CtExport2Pango{_pCtMainWin}.pango_get_from_code_buffer(
+        codebox->get_buffer(), -1, -1, codebox->get_syntax_highlighting());
     while (true)
     {
         // use content if it's ok
@@ -774,8 +793,8 @@ void CtPrint::_process_pango_table(CtPrintData *print_data, CtTable *table, int
 
 Glib::RefPtr<Pango::Layout> CtPrint::_codebox_get_layout(CtCodebox* codebox, Glib::ustring content, Glib::RefPtr<Gtk::PrintContext> context)
 {
-    auto layout = context->create_pango_layout();
-    layout->set_font_description(_code_font);
+    Glib::RefPtr<Pango::Layout> layout = context->create_pango_layout();
+    layout->set_font_description(codebox->get_syntax_highlighting() != CtConst::PLAIN_TEXT_ID ? _code_font : _plain_font);
     double codebox_width = codebox->get_width_in_pixels() ? codebox->get_frame_width() : _text_window_width * codebox->get_frame_width()/100.;
     codebox_width *= _page_dpi_scale;
     if (codebox_width > _page_width)
