@@ -1,7 +1,7 @@
 /*
  * ct_export2txt.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2022
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -41,8 +41,9 @@ Glib::ustring CtExport2Txt::node_export_to_txt(CtTreeIter tree_iter, fs::path fi
     }
     plain_text += selection_export_to_txt(tree_iter, tree_iter.get_node_text_buffer(), sel_start, sel_end, false);
     plain_text += str::repeat(CtConst::CHAR_NEWLINE, 2);
-    if (filepath != "")
-        g_file_set_contents(filepath.c_str(), plain_text.c_str(), (gssize)plain_text.bytes(), nullptr);
+    if (not filepath.empty()) {
+        CtMiscUtil::text_file_set_contents_add_cr_on_win(filepath.string(), plain_text);
+    }
     return plain_text;
 }
 
@@ -71,8 +72,9 @@ void CtExport2Txt::nodes_all_export_to_txt(bool all_tree, fs::path export_dir, f
         if (!all_tree) break;
     }
 
-    if (single_txt_filepath != "")
-        g_file_set_contents(single_txt_filepath.c_str(), tree_plain_text.c_str(), (gssize)tree_plain_text.bytes(), nullptr);
+    if (not single_txt_filepath.empty()) {
+        CtMiscUtil::text_file_set_contents_add_cr_on_win(single_txt_filepath.string(), tree_plain_text);
+    }
 }
 
 // Export the Buffer To Txt
@@ -89,13 +91,13 @@ Glib::ustring CtExport2Txt::selection_export_to_txt(CtTreeIter tree_iter, Glib::
         plain_text += text_slot;
         if (CtTable* ctTable = dynamic_cast<CtTable*>(widget)) plain_text += get_table_plain(ctTable);
         else if (CtCodebox* ctCodebox = dynamic_cast<CtCodebox*>(widget)) plain_text += get_codebox_plain(ctCodebox);
+        else if (CtImageLatex* ctLatex = dynamic_cast<CtImageLatex*>(widget)) plain_text += get_latex_plain(ctLatex);
         start_offset = end_offset;
     }
     plain_text += _plain_process_slot(start_offset, sel_end, text_buffer, check_link_target && widgets.empty());
     return plain_text;
 }
 
-// Returns the plain Table
 Glib::ustring CtExport2Txt::get_table_plain(CtTable* table_orig)
 {
     Glib::ustring table_plain = CtConst::CHAR_NEWLINE;
@@ -109,7 +111,6 @@ Glib::ustring CtExport2Txt::get_table_plain(CtTable* table_orig)
     return table_plain;
 }
 
-// Returns the plain CodeBox
 Glib::ustring CtExport2Txt::get_codebox_plain(CtCodebox* codebox)
 {
     Glib::ustring codebox_plain = CtConst::CHAR_NEWLINE + _pCtMainWin->get_ct_config()->hRule + CtConst::CHAR_NEWLINE;
@@ -118,6 +119,23 @@ Glib::ustring CtExport2Txt::get_codebox_plain(CtCodebox* codebox)
     return codebox_plain;
 }
 
+Glib::ustring CtExport2Txt::get_latex_plain(CtImageLatex* latex)
+{
+    Glib::ustring latex_plain = CtConst::CHAR_NEWLINE + _pCtMainWin->get_ct_config()->hRule + CtConst::CHAR_NEWLINE;
+    Glib::ustring latex_text = latex->get_latex_text();
+    const Glib::ustring::size_type begin_doc = latex_text.find("\\begin{document}");
+    if (std::string::npos != begin_doc) {
+        const Glib::ustring::size_type end_doc = latex_text.rfind("\\end{document}");
+        if (std::string::npos != end_doc and end_doc > begin_doc) {
+            const auto begin_doc2 = begin_doc + 17;
+            latex_text = latex_text.substr(begin_doc2, end_doc - begin_doc2 - 1);
+        }
+    }
+    latex_plain += latex_text;
+    latex_plain += CtConst::CHAR_NEWLINE + _pCtMainWin->get_ct_config()->hRule + CtConst::CHAR_NEWLINE;
+    return latex_plain;
+}
+
 // Process a Single plain Slot
 Glib::ustring CtExport2Txt::_plain_process_slot(int start_offset, int end_offset, Glib::RefPtr<Gtk::TextBuffer> curr_buffer, bool check_link_target)
 {
