@@ -1,7 +1,7 @@
 /*
  * tests_misc_utils.cpp
  *
- * Copyright 2009-2021
+ * Copyright 2009-2022
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -372,6 +372,19 @@ TEST(MiscUtilsGroup, mime__type_contains)
 #endif
 }
 
+TEST(MiscUtilsGroup, text_file_set_contents_add_cr_on_win)
+{
+    fs::path test_file_path = fs::path{UT::unitTestsDataDir} / fs::path{"test_file_line_endings.txt"};
+    const std::string text_file_orig{"one\ntwo\nthree\nfour"};
+#if defined(_WIN32)
+    const std::string text_file_expected{"one\r\ntwo\r\nthree\r\nfour"};
+#else // !_WIN32
+    const std::string text_file_expected{text_file_orig};
+#endif // !_WIN32
+    CtMiscUtil::text_file_set_contents_add_cr_on_win(test_file_path.string(), text_file_orig);
+    ASSERT_STREQ(text_file_expected.c_str(), Glib::file_get_contents(test_file_path.string()).c_str());
+}
+
 TEST(MiscUtilsGroup, parallel_for)
 {
     auto check_range_in_vec = [](const std::vector<int>& vec, const size_t& first, const size_t& last) -> bool {
@@ -430,3 +443,23 @@ TEST(MiscUtilsGroup, get_link_entry)
     ASSERT_STREQ("", CtMiscUtil::get_link_entry("/home/foo/bar").type.c_str());
     ASSERT_STREQ("", CtMiscUtil::get_link_entry("home https://example.com").type.c_str());
 }
+
+TEST(MiscUtilsGroup, get_is_camel_case)
+{
+    Glib::RefPtr<Gtk::TextBuffer> pTextBuffer = Gtk::TextBuffer::create();
+    {
+        const std::string notcamelcase{"notcamelcase"};
+        pTextBuffer->set_text(notcamelcase);
+        ASSERT_FALSE(CtTextIterUtil::get_is_camel_case(pTextBuffer->begin(), notcamelcase.size()));
+    }
+    {
+        const std::string yesCamel1{"yesCamel1"};
+        pTextBuffer->set_text(yesCamel1);
+        ASSERT_TRUE(CtTextIterUtil::get_is_camel_case(pTextBuffer->begin(), yesCamel1.size()));
+    }
+    {
+        const std::string yesCamel2{"YesCamel2"};
+        pTextBuffer->set_text(yesCamel2);
+        ASSERT_TRUE(CtTextIterUtil::get_is_camel_case(pTextBuffer->begin(), yesCamel2.size()));
+    }
+}
